@@ -77,51 +77,38 @@ printscript(systemtap_session& s, ostream& o)
           assert_no_interrupts();
 
           derived_probe* p = s.probes[i];
-          // NB: p->basest() is not so interesting;
-          // p->almost_basest() doesn't quite work, so ...
-          vector<probe*> chain;
-          p->collect_derivation_chain (chain);
-          probe* second = (chain.size()>1) ? chain[chain.size()-2] : chain[0];
-
-          if (s.verbose > 5) {
-          p->printsig(cerr); cerr << endl;
-          cerr << "chain[" << chain.size() << "]:" << endl;
-          for (unsigned j=0; j<chain.size(); j++)
-            {
-              cerr << "  [" << j << "]: " << endl;
-              cerr << "\tlocations[" << chain[j]->locations.size() << "]:" << endl;
-              for (unsigned k=0; k<chain[j]->locations.size(); k++)
-                {
-                  cerr << "\t  [" << k << "]: ";
-                  chain[j]->locations[k]->print(cerr);
-                  cerr << endl;
-                }
-              const probe_alias *a = chain[j]->get_alias();
-              if (a)
-                {
-                  cerr << "\taliases[" << a->alias_names.size() << "]:" << endl;
-                  for (unsigned k=0; k<a->alias_names.size(); k++)
-                    {
-                      cerr << "\t  [" << k << "]: ";
-                      a->alias_names[k]->print(cerr);
-                      cerr << endl;
-                    }
-                }
-            }
+          if (s.verbose > 2) {
+            vector<probe*> chain;
+            p->collect_derivation_chain (chain);
+            p->printsig(cerr); cerr << endl;
+            cerr << "chain[" << chain.size() << "]:" << endl;
+            for (unsigned j=0; j<chain.size(); j++)
+              {
+                cerr << "  [" << j << "]: " << endl;
+                cerr << "\tlocations[" << chain[j]->locations.size() << "]:" << endl;
+                for (unsigned k=0; k<chain[j]->locations.size(); k++)
+                  {
+                    cerr << "\t  [" << k << "]: ";
+                    chain[j]->locations[k]->print(cerr);
+                    cerr << endl;
+                  }
+                const probe_alias *a = chain[j]->get_alias();
+                if (a)
+                  {
+                    cerr << "\taliases[" << a->alias_names.size() << "]:" << endl;
+                    for (unsigned k=0; k<a->alias_names.size(); k++)
+                      {
+                        cerr << "\t  [" << k << "]: ";
+                        a->alias_names[k]->print(cerr);
+                        cerr << endl;
+                      }
+                  }
+              }
           }
-
+          
           stringstream tmps;
-          // XXX PR14297 make this more accurate wrt complex wildcard expansions
-          const probe_point *a = second->get_alias_loc();
-          if (a)
-            {
-              a->print(tmps);
-            }
-          else
-            {
-              assert (second->locations.size() >= 1);
-              second->locations[0]->print(tmps); // XXX: choosing only one location is less arbitrary here than in get_alias_loc(), but still ...
-            }
+          const probe_point *a = p->script_location();
+          a->print(tmps);
           string pp = tmps.str();
 
           // Now duplicate-eliminate.  An alias may have expanded to
@@ -311,10 +298,10 @@ static void*
 sdt_benchmark_thread(void* p)
 {
   unsigned long i = *(unsigned long*)p;
-  PROBE(stap, benchmark-thread-begin);
+  PROBE(stap, benchmark__thread__start);
   while (i--)
     PROBE1(stap, benchmark, i);
-  PROBE(stap, benchmark-thread-end);
+  PROBE(stap, benchmark__thread__end);
   return NULL;
 }
 
@@ -334,7 +321,7 @@ run_sdt_benchmark(systemtap_session& s)
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
 
-  PROBE(stap, benchmark-begin);
+  PROBE(stap, benchmark__start);
 
   pthread_t pthreads[threads];
   for (unsigned long i = 0; i < threads; ++i)
@@ -342,7 +329,7 @@ run_sdt_benchmark(systemtap_session& s)
   for (unsigned long i = 0; i < threads; ++i)
     pthread_join(pthreads[i], NULL);
 
-  PROBE(stap, benchmark-end);
+  PROBE(stap, benchmark__end);
 
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
